#include <vtkActor.h>
#include <vtkAxes.h>
#include <vtkClipPolyData.h>
#include <vtkCubeSource.h>
#include <vtkFillHolesFilter.h>
#include <vtkImplicitBoolean.h>
#include <vtkPlane.h>
#include <vtkPolyDataMapper.h>
#include <vtkPolyDataNormals.h>
#include <vtkProp.h>
#include <vtkProperty.h>
#include <vtkRenderWindowInteractor.h>
#include <vtkRenderer.h>
#include <vtkSmartPointer.h>
#include <vtkTextActor.h>
#include <vtkTextProperty.h>
#include <vtkXOpenGLRenderWindow.h>

#define vtkNew(type, name) vtkSmartPointer<type> name = vtkSmartPointer<type>::New()

int main(void)
{

	/* Axes at origin */
	vtkNew(vtkAxes, axes);
	vtkNew(vtkPolyDataMapper, axesMapper);
	vtkNew(vtkActor, axesActor);
	axesMapper->SetInputConnection(axes->GetOutputPort());
	axesActor->SetMapper(axesMapper);

	/* X, Y, Z */
	vtkNew(vtkTextActor, x2d);
	x2d->SetInput("X");
	x2d->GetPositionCoordinate()->SetCoordinateSystemToWorld();
	x2d->SetPosition(1, 0);
	x2d->GetTextProperty()->SetFontSize(24);
	vtkNew(vtkTextActor, y2d);
	y2d->SetInput("Y");
	y2d->GetPositionCoordinate()->SetCoordinateSystemToWorld();
	y2d->SetPosition(0, 1);
	y2d->GetTextProperty()->SetFontSize(24);
	vtkNew(vtkTextActor, z2d);
	z2d->SetInput("Z");
	z2d->GetPositionCoordinate()->SetCoordinateSystemToWorld();
	z2d->SetPosition(0, 0);
	z2d->GetTextProperty()->SetFontSize(24);

	/* The cube */
	vtkNew(vtkCubeSource, solid);
	solid->SetCenter(0, 0, 0);
	solid->SetXLength(10);
	solid->SetYLength(10);
	solid->SetZLength(10);

	/* Cut planes */
	vtkNew(vtkPlane, plane1);
	plane1->SetOrigin(0, 0, 0);
	plane1->SetNormal(1, -1, 0);

	vtkNew(vtkPlane, plane2);
	plane2->SetOrigin(0, 0, 0);
	plane2->SetNormal(-1, -1, 0);

	/* Boolean for the two planes */
	vtkNew(vtkImplicitBoolean, boolean);
	boolean->AddFunction(plane1);
	boolean->AddFunction(plane2);
	/* boolean->SetOperationTypeTo?????(); */

	/* Clip the polydata */
	vtkNew(vtkClipPolyData, clipper);
	clipper->SetClipFunction(boolean);
	clipper->SetInputConnection(solid->GetOutputPort());

	/* Fill the holes */
	vtkNew(vtkFillHolesFilter, filler);
	filler->SetInput(clipper->GetOutput());
	filler->SetHoleSize(1000);

	/* Fix the normals */
	vtkNew(vtkPolyDataNormals, normals);
	normals->AutoOrientNormalsOn();
	normals->ComputeCellNormalsOn();
	normals->SetInput(filler->GetOutput());

	/* Visualization */
	vtkNew(vtkPolyDataMapper, solidMapper);
	solidMapper->SetInputConnection(filler->GetOutputPort());

	vtkNew(vtkActor, solidActor);
	solidActor->SetMapper(solidMapper);
	solidActor->GetProperty()->SetOpacity(0.6);

	vtkNew(vtkRenderer, renderer);
	renderer->AddViewProp(solidActor);
	renderer->AddViewProp(axesActor);
	renderer->AddViewProp(x2d);
	renderer->AddViewProp(y2d);
	renderer->AddViewProp(z2d);

	vtkNew(vtkXOpenGLRenderWindow, win);
	vtkNew(vtkRenderWindowInteractor, iren);
	win->AddRenderer(renderer);
	iren->SetRenderWindow(win);

	iren->Initialize();
	win->Render();
	iren->Start();

	return (0);
}
